"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


CVVResponseCode = Union[
    Literal[
        "match",
        "no_match",
        "unavailable",
        "not_provided",
    ],
    UnrecognizedStr,
]
