"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GooglePaySessionTypedDict(TypedDict):
    gateway_merchant_id: str
    r"""The gateway ID for the merchant as assigned by our platform."""
    token: str
    r"""The session token for Google Pay."""


class GooglePaySession(BaseModel):
    gateway_merchant_id: str
    r"""The gateway ID for the merchant as assigned by our platform."""

    token: str
    r"""The session token for Google Pay."""
