"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .giftcardrequest import GiftCardRequest, GiftCardRequestTypedDict
from .giftcardstoredrequest import GiftCardStoredRequest, GiftCardStoredRequestTypedDict
from gr4vy.types import BaseModel
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


ItemTypedDict = TypeAliasType(
    "ItemTypedDict", Union[GiftCardStoredRequestTypedDict, GiftCardRequestTypedDict]
)


Item = TypeAliasType("Item", Union[GiftCardStoredRequest, GiftCardRequest])


class GiftCardBalanceRequestTypedDict(TypedDict):
    items: List[ItemTypedDict]
    r"""A list of gift cards to request a balance for."""


class GiftCardBalanceRequest(BaseModel):
    items: List[Item]
    r"""A list of gift cards to request a balance for."""
