"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


Mode = Union[
    Literal[
        "card",
        "redirect",
        "applepay",
        "googlepay",
        "checkout-session",
        "click-to-pay",
        "gift-card",
    ],
    UnrecognizedStr,
]
