"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


TaxIDKind = Union[
    Literal[
        "ae.trn",
        "au.abn",
        "ar.dni",
        "ar.cuil",
        "ar.cuit",
        "br.cnpj",
        "br.cpf",
        "ca.bn",
        "ca.gst_hst",
        "ca.pst_bc",
        "ca.pst_mb",
        "ca.pst_sk",
        "ca.qst",
        "ch.vat",
        "cl.tin",
        "co.itin",
        "co.nit",
        "es.cif",
        "eu.vat",
        "gb.vat",
        "hk.br",
        "id.nik",
        "id.npwp",
        "in.gst",
        "in.pan",
        "jp.cn",
        "jp.rn",
        "kr.brn",
        "li.uid",
        "mx.curp",
        "mx.rfc",
        "my.frp",
        "my.itn",
        "my.nric",
        "my.sst",
        "no.vat",
        "nz.gst",
        "pe.ruc",
        "ph.tin",
        "ru.inn",
        "ru.kpp",
        "sa.vat",
        "sg.gst",
        "sg.uen",
        "th.id",
        "th.vat",
        "tw.vat",
        "us.ein",
        "za.vat",
        "bo.ci",
        "uy.rut",
        "uy.ci",
    ],
    UnrecognizedStr,
]
