"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .forterantifraudoptionscartitembeneficiaryaddress import (
    ForterAntiFraudOptionsCartItemBeneficiaryAddress,
    ForterAntiFraudOptionsCartItemBeneficiaryAddressTypedDict,
)
from .forterantifraudoptionscartitembeneficiarycomments import (
    ForterAntiFraudOptionsCartItemBeneficiaryComments,
    ForterAntiFraudOptionsCartItemBeneficiaryCommentsTypedDict,
)
from .forterantifraudoptionscartitembeneficiarypersonaldetails import (
    ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetails,
    ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetailsTypedDict,
)
from .forterantifraudoptionscartitembeneficiaryphone import (
    ForterAntiFraudOptionsCartItemBeneficiaryPhone,
    ForterAntiFraudOptionsCartItemBeneficiaryPhoneTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import NotRequired, TypedDict


class ForterAntiFraudOptionsCartItemBeneficiaryTypedDict(TypedDict):
    personal_details: ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetailsTypedDict
    address: NotRequired[
        Nullable[ForterAntiFraudOptionsCartItemBeneficiaryAddressTypedDict]
    ]
    r"""Address information of the beneficiary."""
    phone: NotRequired[
        Nullable[List[ForterAntiFraudOptionsCartItemBeneficiaryPhoneTypedDict]]
    ]
    r"""Phone numbers associated with the beneficiary."""
    comments: NotRequired[
        Nullable[ForterAntiFraudOptionsCartItemBeneficiaryCommentsTypedDict]
    ]
    r"""Comments related to the beneficiary."""


class ForterAntiFraudOptionsCartItemBeneficiary(BaseModel):
    personal_details: ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetails

    address: OptionalNullable[ForterAntiFraudOptionsCartItemBeneficiaryAddress] = UNSET
    r"""Address information of the beneficiary."""

    phone: OptionalNullable[List[ForterAntiFraudOptionsCartItemBeneficiaryPhone]] = (
        UNSET
    )
    r"""Phone numbers associated with the beneficiary."""

    comments: OptionalNullable[ForterAntiFraudOptionsCartItemBeneficiaryComments] = (
        UNSET
    )
    r"""Comments related to the beneficiary."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["address", "phone", "comments"]
        nullable_fields = ["address", "phone", "comments"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
