"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class ClickToPaySessionRequestTypedDict(TypedDict):
    checkout_session_id: str
    r"""The checkout session ID to create a Click to Pay session for."""


class ClickToPaySessionRequest(BaseModel):
    checkout_session_id: str
    r"""The checkout session ID to create a Click to Pay session for."""
